package web.multitask.trismegistoservices.utils;

import org.apache.commons.io.FileUtils;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.Base64;

@Service
public class CommonUtils {

    private final String FILE_FOLDER = System.getProperty("java.io.tmpdir");

    public String multipartFileToBase64(MultipartFile file) throws IOException {
        byte[] byteArray = file.getBytes();
        return Base64.getEncoder().encodeToString(byteArray);
    }

    public File base64ToFile(String base64, String name) {
        try {
            byte[] byteArray = Base64.getDecoder().decode(base64);
            String folder = FILE_FOLDER + "/" + name;
            File outputFile = new File(folder);
            FileUtils.writeByteArrayToFile(outputFile, byteArray);
            return outputFile;
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    public String fileToBase64(File file) {
        try {
            byte[] byteArray = FileUtils.readFileToByteArray(file);
            return Base64.getEncoder().encodeToString(byteArray);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    public String byteToBase64(byte[] byteArray) {
        return Base64.getEncoder().encodeToString(byteArray);
    }

    public Resource byteToResource(byte[] byteArray, String name) {
        return new ByteArrayResource(byteArray) {
            @Override
            public String getFilename() {
                return name;
            }
        };
    }

    public Resource fileToResource(File file) {
        return new FileSystemResource(file);
    }

    public Resource base64ToResource(String base64, String name) {
        byte[] byteArray = Base64.getDecoder().decode(base64);
        return new ByteArrayResource(byteArray) {
            @Override
            public String getFilename() {
                return name;
            }
        };
    }

    public boolean deleteAllFiles() {
        try {
            String tmpFolder = System.getProperty("java.io.tmpdir");
            File folder = new File(tmpFolder);
            if (!folder.exists() || !folder.isDirectory()) {
                return false;
            }
            File[] files = folder.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isFile()) {
                        if (!file.delete()) {
                            System.err.println("Failed to delete file: " + file.getAbsolutePath());
                        }
                    }
                }
            }
            return true;
        } catch (SecurityException e) {
            System.err.println("Security exception occurred: " + e.getMessage());
            return false;
        }
    }
}